@@ -158,19 +158,6 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		}
 	}
  	
-	public class ReenableOnClose extends WindowAdapter {
-		JButton but;
-		public ReenableOnClose(JButton b) { but=b; }
-		public void windowClosing(WindowEvent e) {
-			if(but.getActionCommand().equals("raw"))
-				comm.sendInput("!root \"TekkotsuMon\" \"RawCamServer\"");
-			else
-				comm.sendInput("!root \"TekkotsuMon\" \"SegCamServer\"");
-			e.getWindow().dispose();
-			but.setEnabled(true);
-		}
-	}
-
 	void lostConnection() {
 		updated();
 	}
@@ -190,10 +177,15 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 			status.setText("Reconnecting...");
 		} else {
 			if(comm._connectCount>0) {
-				if(comm._connectCount==1)
+				if(comm._connectCount==1) {
 					for(int i=0; i<scriptsModel.getSize(); i++)
 						if(scriptsModel.get(i).toString().equals("STARTUP"))
 							comm.sendInput(((ScriptEntry)scriptsModel.get(i)).cmd);
+					//restart servers for any windows we still have open
+					Iterator mons=comm.dynObjSrcs.values().iterator();
+					while(mons.hasNext())
+						comm.sendInput("!root "+(String)mons.next());
+				}
 				for(int i=0; i<scriptsModel.getSize(); i++)
 					if(scriptsModel.get(i).toString().equals("CONNECT"))
 						comm.sendInput(((ScriptEntry)scriptsModel.get(i)).cmd);
@@ -255,23 +247,25 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		} else if(evt.getSource()==reconnectBut) {
 			int port=comm._port;
 			String addr=comm._host;
+			HashMap dynObjs=comm.dynObjs;
+			HashMap dynObjSrcs=comm.dynObjSrcs;
+			HashMap dynObjPorts=comm.dynObjPorts;
 			comm.kill();
 			comm = new ControllerListener(addr,port);
 			comm.listener=this;
+			comm.dynObjs=dynObjs;
+			comm.dynObjSrcs=dynObjSrcs;
+			comm.dynObjPorts=dynObjPorts;
 			estop.close();
 			estop.open();
 		} else if(evt.getActionCommand().equals("raw") || evt.getActionCommand().equals("rle")) {
-			if(evt.getActionCommand().equals("raw"))
+			if(evt.getActionCommand().equals("raw")) {
 				comm.sendInput("!root \"TekkotsuMon\" \"RawCamServer\"");
-			else
+				comm.dynObjSrcs.put("RawVisionGUI","\"TekkotsuMon\" \"RawCamServer\"");
+			} else {
 				comm.sendInput("!root \"TekkotsuMon\" \"SegCamServer\"");
-			String[] tmp=new String[1];
-			tmp[0]=evt.getActionCommand();
-			VisionGUI vis=new VisionGUI(comm._host,tmp);
-			((JButton)evt.getSource()).setEnabled(false);
-			vis.addWindowListener(new ReenableOnClose((JButton)evt.getSource()));
-			vis.show();
-			vis.toFront();
+				comm.dynObjSrcs.put("SegVisionGUI","\"TekkotsuMon\" \"SegCamServer\"");
+			}
 		} else if(evt.getActionCommand().equals("addbookmark")) {
 			int sel=comm.firstSelected();
 			if(sel==-1 || !comm._isConnected) {
@@ -516,8 +510,11 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		ControllerGUI gui;
 		CloseVisionAdapter(ControllerGUI gui) {this.gui=gui;}
 		public void windowClosing(WindowEvent e) {
-			prefs.putInt("ControllerGUI.location.x",getLocation().x+getInsets().left);
-			prefs.putInt("ControllerGUI.location.y",getLocation().y+getInsets().top);
+			prefs.putInt("ControllerGUI.location.x",getLocation().x);
+			prefs.putInt("ControllerGUI.location.y",getLocation().y);
+			//I think I had needed to add getInsets() to keep the window from moving when reopening it, but now it *causes* it to move... weird.  what changed?
+			//prefs.putInt("ControllerGUI.location.x",getLocation().x+getInsets().left);
+			//prefs.putInt("ControllerGUI.location.y",getLocation().y+getInsets().top);
 			prefs.putInt("ControllerGUI.numScripts",scriptsModel.getSize());
 			for(int i=0; i<scriptsModel.getSize(); i++) {
 				prefs.put("ControllerGUI.script"+i+".title",((ScriptEntry)scriptsModel.get(i)).title);
@@ -525,6 +522,7 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 			}
 			for(int i=0; i<MAX_STORE_INPUT_HIST; i++)
 				prefs.put("inputFieldHistory"+i,(String)inputFieldHistory.get(inputFieldHistory.size()-MAX_STORE_INPUT_HIST+i));
+			gui.comm.removeDynObjs();
 			gui.comm.kill();
 			estop.close();
 			while(gui.comm.isConnected())
